/* Autogenerated by mlir-tblgen; don't manually edit. */
//===----------------------------------------------------------------------===//
// Conversion Group Registration
//===----------------------------------------------------------------------===//

void mlirRegisterConversionPasses() {
  registerConversionPasses();
}

MlirPass mlirCreateConversionArithToLLVMConversionPass() {
  return wrap(createArithToLLVMConversionPass().release());
}
void mlirRegisterConversionArithToLLVMConversionPass() {
  registerArithToLLVMConversionPass();
}


MlirPass mlirCreateConversionConvertAMDGPUToROCDL() {
  return wrap(mlir::createConvertAMDGPUToROCDLPass().release());
}
void mlirRegisterConversionConvertAMDGPUToROCDL() {
  registerConvertAMDGPUToROCDL();
}


MlirPass mlirCreateConversionConvertAffineForToGPU() {
  return wrap(mlir::createAffineForToGPUPass().release());
}
void mlirRegisterConversionConvertAffineForToGPU() {
  registerConvertAffineForToGPU();
}


MlirPass mlirCreateConversionConvertAffineToStandard() {
  return wrap(mlir::createLowerAffinePass().release());
}
void mlirRegisterConversionConvertAffineToStandard() {
  registerConvertAffineToStandard();
}


MlirPass mlirCreateConversionConvertArithToSPIRV() {
  return wrap(mlir::arith::createConvertArithToSPIRVPass().release());
}
void mlirRegisterConversionConvertArithToSPIRV() {
  registerConvertArithToSPIRV();
}


MlirPass mlirCreateConversionConvertArmNeon2dToIntr() {
  return wrap(mlir::createConvertArmNeon2dToIntrPass().release());
}
void mlirRegisterConversionConvertArmNeon2dToIntr() {
  registerConvertArmNeon2dToIntr();
}


MlirPass mlirCreateConversionConvertAsyncToLLVM() {
  return wrap(mlir::createConvertAsyncToLLVMPass().release());
}
void mlirRegisterConversionConvertAsyncToLLVM() {
  registerConvertAsyncToLLVM();
}


MlirPass mlirCreateConversionConvertBufferizationToMemRef() {
  return wrap(mlir::createBufferizationToMemRefPass().release());
}
void mlirRegisterConversionConvertBufferizationToMemRef() {
  registerConvertBufferizationToMemRef();
}


MlirPass mlirCreateConversionConvertComplexToLLVM() {
  return wrap(mlir::createConvertComplexToLLVMPass().release());
}
void mlirRegisterConversionConvertComplexToLLVM() {
  registerConvertComplexToLLVM();
}


MlirPass mlirCreateConversionConvertComplexToLibm() {
  return wrap(mlir::createConvertComplexToLibmPass().release());
}
void mlirRegisterConversionConvertComplexToLibm() {
  registerConvertComplexToLibm();
}


MlirPass mlirCreateConversionConvertComplexToStandard() {
  return wrap(mlir::createConvertComplexToStandardPass().release());
}
void mlirRegisterConversionConvertComplexToStandard() {
  registerConvertComplexToStandard();
}


MlirPass mlirCreateConversionConvertControlFlowToLLVM() {
  return wrap(mlir::cf::createConvertControlFlowToLLVMPass().release());
}
void mlirRegisterConversionConvertControlFlowToLLVM() {
  registerConvertControlFlowToLLVM();
}


MlirPass mlirCreateConversionConvertControlFlowToSPIRV() {
  return wrap(mlir::createConvertControlFlowToSPIRVPass().release());
}
void mlirRegisterConversionConvertControlFlowToSPIRV() {
  registerConvertControlFlowToSPIRV();
}


MlirPass mlirCreateConversionConvertFuncToLLVM() {
  return wrap(mlir::createConvertFuncToLLVMPass().release());
}
void mlirRegisterConversionConvertFuncToLLVM() {
  registerConvertFuncToLLVM();
}


MlirPass mlirCreateConversionConvertFuncToSPIRV() {
  return wrap(mlir::createConvertFuncToSPIRVPass().release());
}
void mlirRegisterConversionConvertFuncToSPIRV() {
  registerConvertFuncToSPIRV();
}


MlirPass mlirCreateConversionConvertGPUToSPIRV() {
  return wrap(mlir::createConvertGPUToSPIRVPass().release());
}
void mlirRegisterConversionConvertGPUToSPIRV() {
  registerConvertGPUToSPIRV();
}


MlirPass mlirCreateConversionConvertGpuLaunchFuncToVulkanLaunchFunc() {
  return wrap(mlir::createConvertGpuLaunchFuncToVulkanLaunchFuncPass().release());
}
void mlirRegisterConversionConvertGpuLaunchFuncToVulkanLaunchFunc() {
  registerConvertGpuLaunchFuncToVulkanLaunchFunc();
}


MlirPass mlirCreateConversionConvertGpuOpsToNVVMOps() {
  return wrap(mlir::createLowerGpuOpsToNVVMOpsPass().release());
}
void mlirRegisterConversionConvertGpuOpsToNVVMOps() {
  registerConvertGpuOpsToNVVMOps();
}


MlirPass mlirCreateConversionConvertGpuOpsToROCDLOps() {
  return wrap(mlir::createLowerGpuOpsToROCDLOpsPass().release());
}
void mlirRegisterConversionConvertGpuOpsToROCDLOps() {
  registerConvertGpuOpsToROCDLOps();
}


MlirPass mlirCreateConversionConvertLinalgToLLVM() {
  return wrap(mlir::createConvertLinalgToLLVMPass().release());
}
void mlirRegisterConversionConvertLinalgToLLVM() {
  registerConvertLinalgToLLVM();
}


MlirPass mlirCreateConversionConvertLinalgToSPIRV() {
  return wrap(mlir::createLinalgToSPIRVPass().release());
}
void mlirRegisterConversionConvertLinalgToSPIRV() {
  registerConvertLinalgToSPIRV();
}


MlirPass mlirCreateConversionConvertLinalgToStandard() {
  return wrap(mlir::createConvertLinalgToStandardPass().release());
}
void mlirRegisterConversionConvertLinalgToStandard() {
  registerConvertLinalgToStandard();
}


MlirPass mlirCreateConversionConvertMathToFuncs() {
  return wrap(mlir::createConvertMathToFuncsPass().release());
}
void mlirRegisterConversionConvertMathToFuncs() {
  registerConvertMathToFuncs();
}


MlirPass mlirCreateConversionConvertMathToLLVM() {
  return wrap(mlir::createConvertMathToLLVMPass().release());
}
void mlirRegisterConversionConvertMathToLLVM() {
  registerConvertMathToLLVM();
}


MlirPass mlirCreateConversionConvertMathToLibm() {
  return wrap(mlir::createConvertMathToLibmPass().release());
}
void mlirRegisterConversionConvertMathToLibm() {
  registerConvertMathToLibm();
}


MlirPass mlirCreateConversionConvertMathToSPIRV() {
  return wrap(mlir::createConvertMathToSPIRVPass().release());
}
void mlirRegisterConversionConvertMathToSPIRV() {
  registerConvertMathToSPIRV();
}


MlirPass mlirCreateConversionConvertMemRefToSPIRV() {
  return wrap(mlir::createConvertMemRefToSPIRVPass().release());
}
void mlirRegisterConversionConvertMemRefToSPIRV() {
  registerConvertMemRefToSPIRV();
}


MlirPass mlirCreateConversionConvertNVGPUToNVVM() {
  return wrap(mlir::createConvertNVGPUToNVVMPass().release());
}
void mlirRegisterConversionConvertNVGPUToNVVM() {
  registerConvertNVGPUToNVVM();
}


MlirPass mlirCreateConversionConvertOpenACCToLLVM() {
  return wrap(mlir::createConvertOpenACCToLLVMPass().release());
}
void mlirRegisterConversionConvertOpenACCToLLVM() {
  registerConvertOpenACCToLLVM();
}


MlirPass mlirCreateConversionConvertOpenACCToSCF() {
  return wrap(mlir::createConvertOpenACCToSCFPass().release());
}
void mlirRegisterConversionConvertOpenACCToSCF() {
  registerConvertOpenACCToSCF();
}


MlirPass mlirCreateConversionConvertOpenMPToLLVM() {
  return wrap(mlir::createConvertOpenMPToLLVMPass().release());
}
void mlirRegisterConversionConvertOpenMPToLLVM() {
  registerConvertOpenMPToLLVM();
}


MlirPass mlirCreateConversionConvertPDLToPDLInterp() {
  return wrap(mlir::createPDLToPDLInterpPass().release());
}
void mlirRegisterConversionConvertPDLToPDLInterp() {
  registerConvertPDLToPDLInterp();
}


MlirPass mlirCreateConversionConvertParallelLoopToGpu() {
  return wrap(mlir::createParallelLoopToGpuPass().release());
}
void mlirRegisterConversionConvertParallelLoopToGpu() {
  registerConvertParallelLoopToGpu();
}


MlirPass mlirCreateConversionConvertSCFToOpenMP() {
  return wrap(mlir::createConvertSCFToOpenMPPass().release());
}
void mlirRegisterConversionConvertSCFToOpenMP() {
  registerConvertSCFToOpenMP();
}


MlirPass mlirCreateConversionConvertSPIRVToLLVM() {
  return wrap(mlir::createConvertSPIRVToLLVMPass().release());
}
void mlirRegisterConversionConvertSPIRVToLLVM() {
  registerConvertSPIRVToLLVM();
}


MlirPass mlirCreateConversionConvertShapeConstraints() {
  return wrap(mlir::createConvertShapeConstraintsPass().release());
}
void mlirRegisterConversionConvertShapeConstraints() {
  registerConvertShapeConstraints();
}


MlirPass mlirCreateConversionConvertShapeToStandard() {
  return wrap(mlir::createConvertShapeToStandardPass().release());
}
void mlirRegisterConversionConvertShapeToStandard() {
  registerConvertShapeToStandard();
}


MlirPass mlirCreateConversionConvertTensorToLinalg() {
  return wrap(mlir::createConvertTensorToLinalgPass().release());
}
void mlirRegisterConversionConvertTensorToLinalg() {
  registerConvertTensorToLinalg();
}


MlirPass mlirCreateConversionConvertTensorToSPIRV() {
  return wrap(mlir::createConvertTensorToSPIRVPass().release());
}
void mlirRegisterConversionConvertTensorToSPIRV() {
  registerConvertTensorToSPIRV();
}


MlirPass mlirCreateConversionConvertVectorToGPU() {
  return wrap(mlir::createConvertVectorToGPUPass().release());
}
void mlirRegisterConversionConvertVectorToGPU() {
  registerConvertVectorToGPU();
}


MlirPass mlirCreateConversionConvertVectorToLLVM() {
  return wrap(mlir::createConvertVectorToLLVMPass().release());
}
void mlirRegisterConversionConvertVectorToLLVM() {
  registerConvertVectorToLLVM();
}


MlirPass mlirCreateConversionConvertVectorToSCF() {
  return wrap(mlir::createConvertVectorToSCFPass().release());
}
void mlirRegisterConversionConvertVectorToSCF() {
  registerConvertVectorToSCF();
}


MlirPass mlirCreateConversionConvertVectorToSPIRV() {
  return wrap(mlir::createConvertVectorToSPIRVPass().release());
}
void mlirRegisterConversionConvertVectorToSPIRV() {
  registerConvertVectorToSPIRV();
}


MlirPass mlirCreateConversionConvertVulkanLaunchFuncToVulkanCalls() {
  return wrap(mlir::createConvertVulkanLaunchFuncToVulkanCallsPass().release());
}
void mlirRegisterConversionConvertVulkanLaunchFuncToVulkanCalls() {
  registerConvertVulkanLaunchFuncToVulkanCalls();
}


MlirPass mlirCreateConversionGpuToLLVMConversionPass() {
  return wrap(mlir::createGpuToLLVMConversionPass().release());
}
void mlirRegisterConversionGpuToLLVMConversionPass() {
  registerGpuToLLVMConversionPass();
}


MlirPass mlirCreateConversionLowerHostCodeToLLVM() {
  return wrap(mlir::createLowerHostCodeToLLVMPass().release());
}
void mlirRegisterConversionLowerHostCodeToLLVM() {
  registerLowerHostCodeToLLVM();
}


MlirPass mlirCreateConversionMapMemRefStorageClass() {
  return wrap(mlir::createMapMemRefStorageClassPass().release());
}
void mlirRegisterConversionMapMemRefStorageClass() {
  registerMapMemRefStorageClass();
}


MlirPass mlirCreateConversionMemRefToLLVMConversionPass() {
  return wrap(createMemRefToLLVMConversionPass().release());
}
void mlirRegisterConversionMemRefToLLVMConversionPass() {
  registerMemRefToLLVMConversionPass();
}


MlirPass mlirCreateConversionReconcileUnrealizedCasts() {
  return wrap(mlir::createReconcileUnrealizedCastsPass().release());
}
void mlirRegisterConversionReconcileUnrealizedCasts() {
  registerReconcileUnrealizedCasts();
}


MlirPass mlirCreateConversionSCFToControlFlow() {
  return wrap(mlir::createConvertSCFToCFPass().release());
}
void mlirRegisterConversionSCFToControlFlow() {
  registerSCFToControlFlow();
}


MlirPass mlirCreateConversionSCFToSPIRV() {
  return wrap(mlir::createConvertSCFToSPIRVPass().release());
}
void mlirRegisterConversionSCFToSPIRV() {
  registerSCFToSPIRV();
}


MlirPass mlirCreateConversionTosaToArith() {
  return wrap(tosa::createTosaToArith().release());
}
void mlirRegisterConversionTosaToArith() {
  registerTosaToArith();
}


MlirPass mlirCreateConversionTosaToLinalg() {
  return wrap(tosa::createTosaToLinalg().release());
}
void mlirRegisterConversionTosaToLinalg() {
  registerTosaToLinalg();
}


MlirPass mlirCreateConversionTosaToLinalgNamed() {
  return wrap(tosa::createTosaToLinalgNamed().release());
}
void mlirRegisterConversionTosaToLinalgNamed() {
  registerTosaToLinalgNamed();
}


MlirPass mlirCreateConversionTosaToSCF() {
  return wrap(tosa::createTosaToSCF().release());
}
void mlirRegisterConversionTosaToSCF() {
  registerTosaToSCF();
}


MlirPass mlirCreateConversionTosaToTensor() {
  return wrap(tosa::createTosaToTensor().release());
}
void mlirRegisterConversionTosaToTensor() {
  registerTosaToTensor();
}

